{-# LANGUAGE FlexibleInstances #-} 
{-# LANGUAGE FlexibleContexts #-}
module Main where

import Control.Monad
import Data.Monoid
import Data.List
import Control.Applicative

data P = P {sr :: Int, ta :: Int, dist :: Int, cost :: Int} deriving Show

type Merge = (Int,Int) -> [P] -> [P] -> [Highway] -> [Highway]

data Highway = Highway {next :: (Int,Int), out :: [Int], weigths :: [(Int, Int, Int)]} deriving Show
--3
initHW :: Int -> Int -> Highway
initHW from l = Highway (from,0) [] (zip3 [ x | x <- [0 .. l-1]] [maxBound | _ <- [0..(l-1)] ] [-1 | _ <- [0..(l-1)] ])

upd :: (Int,Int) -> (Int, Int, Int) -> (Int, Int, Int) -> (Int, Int, Int)
upd (from,new) (a,b,s1) (c,d,s2)
    | a == c && new+b < d = (a,new+b,from)
    | otherwise = (c,d,s2)

trap :: [Int] -> [P] -> [P] -> [P]
trap is ps [] = filter (\p -> or $ map (\i-> ta p == i) is) ps 
trap _ _ ps = ps

mergeDist :: Merge
mergeDist (from,new) ps allP hs = let flatten =  foldl (\ws z -> map (upd (from,new) z) ws) (weigths $ head hs) (zip3 (map ta ps) (map dist ps) (map (\(_,_,a)->a) $ weigths $ head hs))
                                      t = trap ( (map (\(a,_,_)->a) $ filter (\(_,b,_) -> b/=maxBound) (weigths $ head hs)) \\ (from:(out $ head hs))) allP
                                      f = filter (\p -> notElem (ta p) (out $ head hs)) ps
                                      ix = minDist $ t $ f
                                  in (Highway (weightSearch (ta ix) flatten) (from:out (head hs)) flatten):hs

mergeCost :: (Int,Int) -> [P] -> [P] -> [Highway] -> [Highway]
mergeCost (from,new) ps allP hs = let flatten =  foldl (\ws z -> map (upd (from,new) z) ws) (weigths $ head hs) (zip3 (map ta ps) (map cost ps) (map (\(_,_,a)->a) $ weigths $ head hs))
                                      t = trap ((map (\(a,_,_)->a) $ filter (\(_,b,_) -> b/=maxBound) (weigths $ head hs)) \\ (from:(out $ head hs))) allP
                                      f = filter (\p -> notElem (ta p) (out $ head hs)) ps
                                      ix = minCost $ t $ f
                                  in (Highway (weightSearch (ta ix) flatten) (from:out (head hs)) flatten):hs

weightSearch :: Int -> [(Int, Int, Int)] -> (Int, Int)
weightSearch ix ls = (\(a,b,c) -> (a,b)) $ (filter (\l -> (\(a,b,c)->a) l == ix) ls) !! 0 

                       
readAll :: Int -> IO [String]
readAll k 
    | k <= 0 = return []
    | otherwise = do
        linez <- getLine
        fmap (linez :) $ readAll (k-1) 
--0
toP :: [Int] -> P
toP (a:b:c:d:[]) = P a b c d
toP _ = P 0 0 0 0

--1
minDist :: [P] -> P
minDist ps = let mini = minimum $ map dist ps in head $ (filter (\p -> dist p == mini) ps) 

minCost :: [P] -> P
minCost ps = let mini = minimum $ map cost ps in head $ (filter (\p -> cost p == mini) ps)

--2
-- filterSourceByIndex
fsbi :: Int -> [P] -> [P]
fsbi ix ps = filter (\p -> sr p == ix) ps

fstbi :: Int -> Int -> [P] -> [P]
fstbi s t ps = filter (\p -> sr p == s && ta p == t) ps

dijkstra :: Int -> (Int, Int) -> [P] -> [Highway] -> Merge -> [Highway]
dijkstra target t@(a,b) ps hs merge 
        | a == target = hs
        | otherwise = let newHs = merge (next $ head hs) (fsbi (fst $ next $ head hs) ps) ps hs 
                      in dijkstra target (next $ head newHs) ps newHs merge

getPath :: Int -> Int -> [Highway] -> [Int] 
getPath s t [] = [s]
getPath s t (h:hs) 
    | t == (fst $ next h) && (t /= s) = let k = ((\(a,b,c)->c) . (\x -> if length x > 0 then head x else (-999,-999,-999))) $ filter (\(a,b,c) -> a == t) $ weigths h 
                                      in (getPath s k hs) ++ [t]
    | otherwise = getPath s t hs

total :: [Int] -> [P] -> (P -> Int) -> Int
total (x:y:xs) ps f = go + (total (y:xs) ps f)
    where
        go = f . head $ fstbi x y ps
total _ ps f = 0

doubleH :: P -> P
doubleH (P s t d c) = P t s d c 

stri :: Char -> Char 
stri '[' = ' '
stri ']' = ' '
stri ',' = ' '
stri x = x

cross :: [([Int],Int,Int)] -> [([Int],Int,Int)] -> [([Int],Int,Int)]
cross (l:ls) (r:rs) = [r] ++ [l] ++ (cross ls rs) 
cross _ _ = []

main :: IO ()
main = do
    firstLine <- fmap words getLine
    cities <- (return $ read $ firstLine !! 0) :: IO Int
    paths <- (return $ read $ firstLine !! 1) :: IO Int
    listPathsStr <- readAll paths
    queries <- readLn :: IO Int
    queryList' <- readAll queries
    citiesList' <- return $ map toP $ map (map read . words) listPathsStr :: IO [P] 
    citiesList <- return $ citiesList' ++ (map doubleH citiesList')
    queryList <- return $ map (map read . words) queryList' :: IO [[Int]] 
    highsDists <- return $ map (\q -> dijkstra (q !! 1) ((q !! 0),maxBound) citiesList [(initHW (q !! 0) cities)] mergeDist) queryList
    highsCosts <- return $ map (\q -> dijkstra (q !! 1) ((q !! 0),maxBound) citiesList [(initHW (q !! 0) cities)] mergeCost) queryList
    z <- return $ zip highsDists queryList
    z1 <- return $ zip highsCosts queryList
    pathDist <- return $ map (\(h,q) -> getPath (q !! 0) (q !! 1) h) z
    pathCost <- return $ map (\(h,q) -> getPath (q !! 0) (q !! 1) h) z1
    minDists <- return $ map (\h -> snd $ next $ head h) highsDists
    minCosts <- return $ map (\h -> snd $ next $ head h) highsCosts
    costs <- return $ map (\x -> total x citiesList cost) pathDist
    dists <- return $ map (\x -> total x citiesList dist) pathCost
    zcd <- return $ zip3 pathDist minDists costs
    zcc <- return $ zip3 pathCost dists minCosts
    last <- return $ cross zcd zcc
    mapM_ (\(p,m,c) -> putStrLn (show m ++ " " ++ show c) >> (putStrLn $ map stri $ tail $ show p)) last
    